use std::io::BufReader;
use std::fs::File;

use anyhow::{Context, Result};
use handlebars::Handlebars;
use serde_derive::Deserialize;
use serde_json::json;

fn default_ldap_url() -> String {
    "ldap://localhost".to_string()
}

fn default_host() -> String {
    "localhost".to_string()
}

fn default_port() -> u16 {
    8000
}

#[derive(Deserialize)]
pub struct Config {
    pub dn: String,

    #[serde(default = "default_ldap_url")]
    pub ldap_url: String,

    #[serde(default = "default_host")]
    pub host: String,

    #[serde(default = "default_port")]
    pub port: u16,
}

pub fn load_config(file_path: &str) -> Result<Config> {
    let file = File::open(file_path).
        with_context(|| format!("Failed to open configuration file '{}'", file_path))?;

    let config: Config = serde_json::from_reader(BufReader::new(file))
        .with_context(|| format!("Failed to parse configuration file '{}'", file_path))?;

    Handlebars::new()
        .render_template(&config.dn, &json!({"username" : "foo"}))
        .map(|_| ())
        .with_context(|| format!("Failed to parse DN ({}) in configuration file '{}'", config.dn, file_path))?;

    Ok(config)
}
